#region usings
using System;
using System.ComponentModel.Composition;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;
#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "StateLogic", Category = "Value", Help = "Handle the screen state.", Tags = "")]
	#endregion PluginInfo
	public class ValueStateLogicNode : IPluginEvaluate
	{
		#region fields & pins
		
		// The size of the contour coming in from the camera.
		[Input("ContourArea", DefaultValue = 0.0)]
		IDiffSpread<double> ContourArea;

		// The desired screen state as an int output pin.
		[Output("CurrentScreenState")]
		ISpread<int> CurrentScreenState;

		[Output("IsCameraEnabled")]
		ISpread<bool> IsCameraEnabled;

		[Output("CreateBang")]
		ISpread<bool> CreateBang;

		[Output("CreatePath")]
		ISpread<bool> CreatePath;
		
		// Access to the log window.
		[Import()]
		ILogger FLogger;
		
		#endregion fields & pins

		// The current screen state, internal to the class.
		ScreenState CurrentState = ScreenState.Attract;

		// The last time the screen state changed, for changing states after a delay.
		DateTime LastStateChange = DateTime.Now;
		
		// A count of how many frames have had a contour in attract mode, or not had a contour in active mode.
		int ConsecutiveAreaFrames = 0;

		// How long to spend on the active screen before taking the snapshot.
		// (Might read this from a command-line arg.)
		static TimeSpan ActiveToSnapshotDelay = TimeSpan.FromMilliseconds(1500);
		
		// How long to spend on the snapshot screen before switching to recharge.
		// (Might read this from a command-line arg.)
		static TimeSpan SnapshotToRechargeDelay = TimeSpan.FromMilliseconds(2000);
		
		// How long to spend on the recharge screen before switching to attract.
		// (Might replace this with a command from the server to go back to attract.)
		static TimeSpan RechargeToAttractDelay = TimeSpan.FromMilliseconds(7000);
		
		// How many frames an area must appear or disappear for before a state change is triggered.
		// (Might read this from a command-line arg.)
		static int ConsecutiveFramesToChange = 20;
		
		// Called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			ScreenState newState = CurrentState;
			double area = ContourArea.SliceCount == 1 ? ContourArea[0] : 0;
			TimeSpan timeSinceLastChange = DateTime.Now - LastStateChange;
			IsCameraEnabled.SliceCount = 1;
			IsCameraEnabled[0] = true;
			CreateBang[0] = false;
			CreatePath[0] = false;

			switch(CurrentState)
			{
				case ScreenState.Attract:
					if(area > 0)
					{
						ConsecutiveAreaFrames++;
						if(ConsecutiveAreaFrames >= ConsecutiveFramesToChange)
						{
							newState = ScreenState.Active;
							IsCameraEnabled[0] = true;
							IsCameraEnabled.SliceCount = 1;
						}
					}

					break;
				case ScreenState.Active:
					if(area == 0)
					{
						ConsecutiveAreaFrames++;
						if(ConsecutiveAreaFrames >= ConsecutiveFramesToChange)
						{
							newState = ScreenState.Attract;
							IsCameraEnabled[0] = true;
							IsCameraEnabled.SliceCount = 1;
						}
					}
					else if(timeSinceLastChange > ActiveToSnapshotDelay)
					{
						newState = ScreenState.Snapshot;
					}
				
					break;
				case ScreenState.Snapshot:
					if(timeSinceLastChange > SnapshotToRechargeDelay)
					{
						newState = ScreenState.Recharge;
						CreateBang[0] = true;
					}
				
					break;
				case ScreenState.Recharge:
					if(timeSinceLastChange > RechargeToAttractDelay)
					{
						newState = ScreenState.Attract;
					}
				
					break;
			}
			
			if(newState != CurrentState)
			{
				FLogger.Log(LogType.Debug, newState.ToString() + " " + ConsecutiveAreaFrames.ToString());
				CurrentState = newState;
				LastStateChange = DateTime.Now;
				CurrentScreenState[0] = (int)CurrentState;
				ConsecutiveAreaFrames = 0;
			}
		}
	}
	
	internal enum ScreenState
	{
		// Shown when nothing is in front of the camera.
		Attract,
		
		// Shown when a shape is in front of the camera.
		Active,
		
		// Shown after a snapshot is taken.
		Snapshot,
		
		// Shown to introduce a delay between snapshots.
		Recharge
	}
}